program YTD;
{$APPTYPE CONSOLE}
uses
  SysUtils,
  uYouTube in 'uYouTube.pas';

const Version = '0.03';

type
  TMyYouTube = class(TYouTube)
    protected
      LastProc: int64;
      ProgressBarLen: integer;
      ProgressBar: string;
      procedure DoProgress(Step: TYouTubeProgress; const Data: string; Progress, Max: int64); override;
    public
      function Download(const Url: string): boolean; override;
    end;

var You: TMyYouTube;
    i: integer;

{ TMyYouTube }

function TMyYouTube.Download(const Url: string): boolean;
begin
  LastProc := -1;
  ProgressBarLen := 40;
  ProgressBar := StringOfChar(' ', ProgressBarLen);
  Result := inherited Download(Url);
end;

procedure TMyYouTube.DoProgress(Step: TYouTubeProgress; const Data: string; Progress, Max: int64);
var Proc: int64;
    i, n: integer;
begin
  inherited;
  case Step of
    ytpDownloadingVideoPage:
      Writeln('  - downloading video page');
    ytpFoundTitle:
      Writeln('  - video title: ' + Data);
    ytpDownloadError:
      Writeln('  - failed to download: ' + Data);
    ytpParamsError:
      Writeln('  - failed to locate video parameters');
    ytpDownloadingVideo:
      begin
      if Progress = 0 then
        begin
        Writeln('  - downloading video data');
        end;
      if Max > 1 then
        begin
        Proc := 1000 * Progress div Max;
        if Proc <> LastProc then
          begin
          LastProc := Proc;
          n := Proc div (1000 div ProgressBarLen);
          for i := 1 to ProgressBarLen do
            if i <= n then
              ProgressBar[i] := '#'
            else
              ProgressBar[i] := ' ';
          Write(Format('    <%s> %d.%d%% (%d/%d)'#13, [ProgressBar, Proc div 10, Proc mod 10, Progress, Max]));
          end;
        end;
      end;
    ytpFinished:
      Writeln('  - done.' + StringOfChar(' ', 70));
    end;
end;

begin
  You := TMyYouTube.Create;
  try
    Writeln('YouTube Downloader v' + VERSION);
    Writeln('------------------------------------------------------------------------------');
    if ParamCount <= 0 then
      begin
      Writeln('Syntax: ' + ExtractFileName(ParamStr(0)) + ' <url> [<url>] [<url>] ...');
      end
    else
      begin
      for i := 1 to ParamCount do
        begin
        Writeln('Downloading: ' + ParamStr(i));
        try
          if not You.Download(ParamStr(i)) then
            Writeln('ERROR while ' + LastGoodStatusStr[You.LastGoodStatus]);
        except
          on E: Exception do
            begin
            Writeln('ERROR ' + E.ClassName + ' while ' + LastGoodStatusStr[You.LastGoodStatus]);
            Writeln('  Message: ' + E.Message);
            end;
          end;
        end;
      end;
  finally
    You.Free;
    end;
end.
